import { OperationContentsAndResult, OperationContentsAndResultReveal } from '@taquito/rpc';
import { Context } from '../context';
import { ForgedBytes } from './types';
/**
 * @description Utility class to interact with Tezos operations
 */
export declare class Operation {
    readonly hash: string;
    readonly raw: ForgedBytes;
    readonly results: OperationContentsAndResult[];
    protected readonly context: Context;
    private _pollingConfig$;
    private lastHead;
    private currentHead$;
    private confirmed$;
    protected _foundAt: number;
    get includedInBlock(): number;
    /**
     *
     * @param hash Operation hash
     * @param raw Raw operation that was injected
     * @param context Taquito context allowing access to rpc and signer
     * @throws {@link InvalidOperationHashError}
     */
    constructor(hash: string, raw: ForgedBytes, results: OperationContentsAndResult[], context: Context);
    get revealOperation(): false | OperationContentsAndResultReveal | undefined;
    get revealStatus(): import("@taquito/rpc").OperationResultStatusEnum | "unknown";
    get status(): import("@taquito/rpc").OperationResultStatusEnum | "unknown";
    /**
     *
     * @param confirmations [0] Number of confirmation to wait for
     * @param timeout [180] Timeout
     */
    confirmation(confirmations?: number, timeout?: number): Promise<number>;
}
