import { OperationContentsAndResult, OperationContentsAndResultTransaction, OperationContentsTransaction } from '@taquito/rpc';
import BigNumber from 'bignumber.js';
import { Context } from '../context';
import { Operation } from './operations';
import { FeeConsumingOperation, ForgedBytes, GasConsumingOperation, StorageConsumingOperation } from './types';
/**
 * @description Transaction operation provides utility functions to fetch a newly issued transaction
 *
 * @warn Currently supports one transaction per operation
 */
export declare class TransactionOperation extends Operation implements GasConsumingOperation, StorageConsumingOperation, FeeConsumingOperation {
    private readonly params;
    readonly source: string;
    constructor(hash: string, params: OperationContentsTransaction, source: string, raw: ForgedBytes, results: OperationContentsAndResult[], context: Context);
    get operationResults(): OperationContentsAndResultTransaction[];
    get status(): import("@taquito/rpc").OperationResultStatusEnum | "unknown";
    get amount(): BigNumber;
    get destination(): string;
    get fee(): number;
    get gasLimit(): number;
    get storageLimit(): number;
    private sumProp;
    get consumedGas(): string;
    get consumedMilliGas(): string;
    get storageDiff(): string;
    get storageSize(): string;
    get errors(): import("@taquito/rpc").TezosGenericOperationError[];
}
